@@ -50,6 +50,10 @@ xrdp_rdp_read_config(const char *xrdp_ini, struct xrdp_client_info *client_info)
     char *tmp = NULL;
     int tmp_length = 0;
 
+    client_info->xrdp_keyboard_overrides.type = -1;
+    client_info->xrdp_keyboard_overrides.subtype = -1;
+    client_info->xrdp_keyboard_overrides.layout = -1;
+
     /* initialize (zero out) local variables: */
     items = list_create();
     items->auto_free = 1;
@@ -278,6 +282,18 @@ xrdp_rdp_read_config(const char *xrdp_ini, struct xrdp_client_info *client_info)
         {
             g_strncpy(client_info->domain_user_separator, value, sizeof(client_info->domain_user_separator) - 1);
         }
+        else if (g_strcasecmp(item, "xrdp.override_keyboard_type") == 0)
+        {
+            client_info->xrdp_keyboard_overrides.type = g_atoix(value);
+        }
+        else if (g_strcasecmp(item, "xrdp.override_keyboard_subtype") == 0)
+        {
+            client_info->xrdp_keyboard_overrides.subtype = g_atoix(value);
+        }
+        else if (g_strcasecmp(item, "xrdp.override_keylayout") == 0)
+        {
+            client_info->xrdp_keyboard_overrides.layout = g_atoix(value);
+        }
     }
 
     list_delete(items);
@@ -611,7 +627,7 @@ xrdp_rdp_send_data(struct xrdp_rdp *self, struct stream *s,
             ls.data = mppc_enc->outputBuffer - (rdp_offset + 18);
             ls.p = ls.data + rdp_offset;
             ls.end = ls.p + clen;
-            ls.size = clen;
+            ls.size = s->end - s->data;
             ls.iso_hdr = ls.data + iso_offset;
             ls.mcs_hdr = ls.data + mcs_offset;
             ls.sec_hdr = ls.data + sec_offset;
