@@ -94,10 +94,87 @@ lxrdp_start(struct mod *mod, int w, int h, int bpp)
 
 /******************************************************************************/
 /* return error */
+static void
+set_keyboard_overrides(struct mod *mod)
+{
+    const struct kbd_overrides *ko = &mod->kbd_overrides;
+    rdpSettings *settings = mod->inst->settings;
+
+    if (mod->allow_client_kbd_settings)
+    {
+        settings->kbd_type = mod->client_info.keyboard_type;
+        settings->kbd_subtype = mod->client_info.keyboard_subtype;
+        /* Define the most common number of function keys, 12.
+           because we can't get it from client. */
+        settings->kbd_fn_keys = 12;
+        settings->kbd_layout = mod->client_info.keylayout;
+
+        /* Exception processing for each RDP Keyboard type */
+        if (mod->client_info.keyboard_type == 0x00)
+        {
+            /* 0x00000000 : Set on Server */
+            LOG(LOG_LEVEL_WARNING, "keyboard_type:[0x%02x] ,Set on Server",
+                mod->client_info.keyboard_type);
+        }
+        else if (mod->client_info.keyboard_type == 0x04)
+        {
+            /* 0x00000004 : IBM enhanced (101- or 102-key) keyboard */
+            /* Nothing to do. */
+        }
+        else if (mod->client_info.keyboard_type == 0x07)
+        {
+            /* 0x00000007 : Japanese keyboard */
+            /* Nothing to do. */
+        }
+    }
+
+    if (ko->type != 0)
+    {
+        LOG(LOG_LEVEL_INFO, "overrode kbd_type 0x%02X with 0x%02X",
+            settings->kbd_type, ko->type);
+        settings->kbd_type = ko->type;
+    }
+
+    if (ko->subtype != 0)
+    {
+        LOG(LOG_LEVEL_INFO, "overrode kbd_subtype 0x%02X with 0x%02X",
+            settings->kbd_subtype, ko->subtype);
+        settings->kbd_subtype = ko->subtype;
+    }
+
+    if (ko->fn_keys != 0)
+    {
+        LOG(LOG_LEVEL_INFO, "overrode kbd_fn_keys %d with %d",
+            settings->kbd_fn_keys, ko->fn_keys);
+        settings->kbd_fn_keys = ko->fn_keys;
+    }
+
+    if (ko->layout != 0)
+    {
+        LOG(LOG_LEVEL_INFO, "overrode kbd_layout 0x%08X with 0x%08X",
+            settings->kbd_layout, ko->layout);
+        settings->kbd_layout = ko->layout;
+    }
+
+    if (ko->layout_mask != 0)
+    {
+        LOG(LOG_LEVEL_INFO, "Masked kbd_layout 0x%08X to 0x%08X",
+            settings->kbd_layout, settings->kbd_layout & ko->layout_mask);
+        settings->kbd_layout &= ko->layout_mask;
+    }
+
+    LOG(LOG_LEVEL_INFO, "NeutrinoRDP proxy remote keyboard settings, "
+        "kbd_type:[0x%02X], kbd_subtype:[0x%02X], "
+        "kbd_fn_keys:[%02d], kbd_layout:[0x%08X]",
+        settings->kbd_type, settings->kbd_subtype,
+        settings->kbd_fn_keys, settings->kbd_layout);
+}
+
 static int
 lxrdp_connect(struct mod *mod)
 {
     boolean ok;
+    set_keyboard_overrides(mod);
 
     LOG_DEVEL(LOG_LEVEL_TRACE, "lxrdp_connect:");
 
@@ -182,18 +259,30 @@ lxrdp_connect(struct mod *mod)
         }
 
 #endif
-        LOG(LOG_LEVEL_ERROR, "freerdp_connect Failed to "
-            "destination :%s:%d",
+        LOG(LOG_LEVEL_ERROR, "NeutrinoRDP proxy connection: status [Failed],"
+            " RDP client [%s:%s], RDP server [%s:%d], RDP server username [%s],"
+            " xrdp pamusername [%s], xrdp process id [%d]",
+            mod->client_info.client_addr,
+            mod->client_info.client_port,
             mod->inst->settings->hostname,
-            mod->inst->settings->port);
+            mod->inst->settings->port,
+            mod->inst->settings->username,
+            mod->pamusername,
+            g_getpid());
         return 1;
     }
     else
     {
-        LOG(LOG_LEVEL_INFO, "freerdp_connect returned Success to "
-            "destination :%s:%d",
+        LOG(LOG_LEVEL_INFO, "NeutrinoRDP proxy connection: status [Success],"
+            " RDP client [%s:%s], RDP server [%s:%d], RDP server username [%s],"
+            " xrdp pamusername [%s], xrdp process id [%d]",
+            mod->client_info.client_addr,
+            mod->client_info.client_port,
             mod->inst->settings->hostname,
-            mod->inst->settings->port);
+            mod->inst->settings->port,
+            mod->inst->settings->username,
+            mod->pamusername,
+            g_getpid());
     }
 
     return 0;
@@ -433,6 +522,16 @@ lxrdp_end(struct mod *mod)
     }
 
     LOG_DEVEL(LOG_LEVEL_DEBUG, "lxrdp_end:");
+    LOG(LOG_LEVEL_INFO, "NeutrinoRDP proxy connection: status [Disconnect],"
+        " RDP client [%s:%s], RDP server [%s:%d], RDP server username [%s],"
+        " xrdp pamusername [%s], xrdp process id [%d]",
+        mod->client_info.client_addr,
+        mod->client_info.client_port,
+        mod->inst->settings->hostname,
+        mod->inst->settings->port,
+        mod->inst->settings->username,
+        mod->pamusername,
+        g_getpid());
     return 0;
 }
 
@@ -467,6 +566,7 @@ lxrdp_set_param(struct mod *mod, const char *name, const char *value)
     }
     else if (g_strcmp(name, "keylayout") == 0)
     {
+        LOG(LOG_LEVEL_DEBUG, "%s:[0x%08X]", name, g_atoi(value));
     }
     else if (g_strcmp(name, "name") == 0)
     {
@@ -504,12 +604,115 @@ lxrdp_set_param(struct mod *mod, const char *name, const char *value)
     {
         settings->desktop_resize = g_text2bool(value);
     }
-    else if (g_strcmp(name, "pamusername") == 0 ||
-             g_strcmp(name, "pampassword") == 0 ||
-             g_strcmp(name, "pammsessionmng") == 0)
+    else if (g_strcmp(name, "pamusername") == 0)
+    {
+        g_strncpy(mod->pamusername, value, 255);
+    }
+    else if (g_strcmp(name, "pampassword") == 0 ||
+             g_strcmp(name, "pamsessionmng") == 0)
     {
         /* Valid (but unused) parameters not logged */
     }
+    else if (g_strcmp(name, "channel.rdpdr") == 0 ||
+             g_strcmp(name, "channel.rdpsnd") == 0 ||
+             g_strcmp(name, "channel.cliprdr") == 0 ||
+             g_strcmp(name, "channel.drdynvc") == 0)
+    {
+        /* Valid (but unused) parameters not logged */
+    }
+    else if (g_strcmp(name, "perf.allow_client_experiencesettings") == 0)
+    {
+        mod->allow_client_experiencesettings = g_text2bool(value);
+    }
+    else if (g_strcmp(name, "perf.wallpaper") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_WALLPAPER;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_WALLPAPER;
+        }
+    }
+    else if (g_strcmp(name, "perf.font_smoothing") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_ENABLE_FONT_SMOOTHING;
+        if (g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_ENABLE_FONT_SMOOTHING;
+        }
+    }
+    else if (g_strcmp(name, "perf.desktop_composition") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_ENABLE_DESKTOP_COMPOSITION;
+        if (g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_ENABLE_DESKTOP_COMPOSITION;
+        }
+    }
+    else if (g_strcmp(name, "perf.full_window_drag") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_FULLWINDOWDRAG;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_FULLWINDOWDRAG;
+        }
+    }
+    else if (g_strcmp(name, "perf.menu_anims") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_MENUANIMATIONS;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_MENUANIMATIONS;
+        }
+    }
+    else if (g_strcmp(name, "perf.themes") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_THEMING;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_THEMING;
+        }
+    }
+    else if (g_strcmp(name, "perf.cursor_blink") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_CURSORSETTINGS;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_CURSORSETTINGS;
+        }
+    }
+    else if (g_strcmp(name, "perf.cursor_shadow") == 0)
+    {
+        mod->perf_settings_override_mask |= PERF_DISABLE_CURSOR_SHADOW;
+        if (!g_text2bool(value))
+        {
+            mod->perf_settings_values_mask |= PERF_DISABLE_CURSOR_SHADOW;
+        }
+    }
+    else if (g_strcmp(name, "neutrinordp.allow_client_keyboardLayout") == 0)
+    {
+        mod->allow_client_kbd_settings = g_text2bool(value);
+    }
+    else if (g_strcmp(name, "neutrinordp.override_keyboardLayout_mask") == 0)
+    {
+        /* Keyboard values are stored for later processing */
+        mod->kbd_overrides.layout_mask = g_atoix(value);
+    }
+    else if (g_strcmp(name, "neutrinordp.override_kbd_type") == 0)
+    {
+        mod->kbd_overrides.type = g_atoix(value);
+    }
+    else if (g_strcmp(name, "neutrinordp.override_kbd_subtype") == 0)
+    {
+        mod->kbd_overrides.subtype = g_atoix(value);
+    }
+    else if (g_strcmp(name, "neutrinordp.override_kbd_fn_keys") == 0)
+    {
+        mod->kbd_overrides.fn_keys = g_atoix(value);
+    }
+    else if (g_strcmp(name, "neutrinordp.override_kbd_layout") == 0)
+    {
+        mod->kbd_overrides.layout = g_atoix(value);
+    }
     else
     {
         LOG(LOG_LEVEL_WARNING, "lxrdp_set_param: unknown name [%s] value [%s]", name, value);
@@ -1650,6 +1853,60 @@ lfreerdp_pre_connect(freerdp *instance)
         // | PERF_DISABLE_CURSOR_SHADOW | PERF_DISABLE_CURSORSETTINGS;
     }
 
+    /* Allow users or administrators to configure the mstsc experience settings. #1903 */
+
+    if ((mod->allow_client_experiencesettings == 1) &&
+            (mod->client_info.mcs_connection_type == CONNECTION_TYPE_AUTODETECT))
+    {
+        /* auto-detect not yet supported - use default performance settings */
+    }
+    else if (mod->allow_client_experiencesettings == 1)
+    {
+        instance->settings->performance_flags =
+            (mod->client_info.rdp5_performanceflags &
+             /* Mask to avoid accepting invalid flags. */
+             (PERF_DISABLE_WALLPAPER |
+              PERF_DISABLE_FULLWINDOWDRAG |
+              PERF_DISABLE_MENUANIMATIONS |
+              PERF_DISABLE_THEMING |
+              PERF_DISABLE_CURSOR_SHADOW |
+              PERF_DISABLE_CURSORSETTINGS |
+              PERF_ENABLE_FONT_SMOOTHING |
+              PERF_ENABLE_DESKTOP_COMPOSITION));
+
+        LOG(LOG_LEVEL_DEBUG, "RDP client experience settings, "
+            "rdp5_performance_flags:[0x%08x], "
+            "masked performance_flags:[0x%08x]",
+            mod->client_info.rdp5_performanceflags,
+            instance->settings->performance_flags);
+
+        if (mod->client_info.rail_enable &&
+                (mod->client_info.rail_support_level > 0))
+        {
+            instance->settings->performance_flags |= (PERF_DISABLE_WALLPAPER |
+                    PERF_DISABLE_FULLWINDOWDRAG);
+            LOG(LOG_LEVEL_DEBUG, "Add in performance setting for Railsupport:"
+                "[0x%08x]", PERF_DISABLE_WALLPAPER |
+                PERF_DISABLE_FULLWINDOWDRAG);
+        }
+    }
+
+    LOG(LOG_LEVEL_DEBUG, "before overriding performance_flags:[0x%08x]",
+        instance->settings->performance_flags);
+    LOG(LOG_LEVEL_DEBUG, "perf_settings_override_mask:[0x%08x], "
+        "perf_settings_values_mask:[0x%08x]",
+        mod->perf_settings_override_mask,
+        mod->perf_settings_values_mask);
+
+    /* Clear bits for any overridden performance settings */
+    instance->settings->performance_flags &= ~mod->perf_settings_override_mask;
+
+    /* Add in overridden performance settings */
+    instance->settings->performance_flags |= mod->perf_settings_values_mask;
+
+    LOG(LOG_LEVEL_DEBUG, "final performance_flags:[0x%08x]",
+        instance->settings->performance_flags);
+
     instance->settings->compression = 0;
     instance->settings->ignore_certificate = 1;
 
