@@ -33,62 +33,19 @@
 #include "file.h"
 #include "sesman.h"
 #include "log.h"
+#include "string_calls.h"
 
-
-
-/******************************************************************************/
-int
-config_read(struct config_sesman *cfg)
-{
-    int fd;
-    struct list *sec;
-    struct list *param_n;
-    struct list *param_v;
-    char cfg_file[256];
-
-    g_snprintf(cfg_file, 255, "%s/sesman.ini", XRDP_CFG_PATH);
-    fd = g_file_open(cfg_file);
-
-    if (-1 == fd)
-    {
-        return 1;
-    }
-
-    g_memset(cfg, 0, sizeof(struct config_sesman));
-    sec = list_create();
-    sec->auto_free = 1;
-    file_read_sections(fd, sec);
-    param_n = list_create();
-    param_n->auto_free = 1;
-    param_v = list_create();
-    param_v->auto_free = 1;
-
-    /* read global config */
-    config_read_globals(fd, cfg, param_n, param_v);
-
-    /* read Xvnc/X11rdp/Xorg parameter list */
-    config_read_vnc_params(fd, cfg, param_n, param_v);
-    config_read_rdp_params(fd, cfg, param_n, param_v);
-    config_read_xorg_params(fd, cfg, param_n, param_v);
-
-    /* read security config */
-    config_read_security(fd, &(cfg->sec), param_n, param_v);
-
-    /* read session config */
-    config_read_sessions(fd, &(cfg->sess), param_n, param_v);
-
-    config_read_session_variables(fd, cfg, param_n, param_v);
-
-    /* cleanup */
-    list_delete(sec);
-    list_delete(param_v);
-    list_delete(param_n);
-    g_file_close(fd);
-    return 0;
-}
-
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [global] configuration section
+ * @param file configuration file descriptor
+ * @param cf pointer to a config struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_globals(int file, struct config_sesman *cf, struct list *param_n,
                     struct list *param_v)
 {
@@ -205,8 +162,17 @@ config_read_globals(int file, struct config_sesman *cf, struct list *param_n,
     return 0;
 }
 
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [Security] configuration section
+ * @param file configuration file descriptor
+ * @param sc pointer to a config_security struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_security(int file, struct config_security *sc,
                      struct list *param_n,
                      struct list *param_v)
@@ -273,8 +239,17 @@ config_read_security(int file, struct config_security *sc,
     return 0;
 }
 
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [Sessions] configuration section
+ * @param file configuration file descriptor
+ * @param ss pointer to a config_sessions struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_sessions(int file, struct config_sessions *se, struct list *param_n,
                      struct list *param_v)
 {
@@ -360,8 +335,17 @@ config_read_sessions(int file, struct config_sessions *se, struct list *param_n,
     return 0;
 }
 
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [X11rdp] configuration section
+ * @param file configuration file descriptor
+ * @param cs pointer to a config_sesman struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_rdp_params(int file, struct config_sesman *cs, struct list *param_n,
                        struct list *param_v)
 {
@@ -383,8 +367,17 @@ config_read_rdp_params(int file, struct config_sesman *cs, struct list *param_n,
     return 0;
 }
 
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [Xorg] configuration section
+ * @param file configuration file descriptor
+ * @param cs pointer to a config_sesman struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_xorg_params(int file, struct config_sesman *cs,
                         struct list *param_n, struct list *param_v)
 {
@@ -407,8 +400,17 @@ config_read_xorg_params(int file, struct config_sesman *cs,
     return 0;
 }
 
-/******************************************************************************/
-int
+/***************************************************************************//**
+ *
+ * @brief Reads sesman [Xvnc] configuration section
+ * @param file configuration file descriptor
+ * @param cs pointer to a config_sesman struct
+ * @param param_n parameter name list
+ * @param param_v parameter value list
+ * @return 0 on success, 1 on failure
+ *
+ */
+static int
 config_read_vnc_params(int file, struct config_sesman *cs, struct list *param_n,
                        struct list *param_v)
 {
@@ -431,7 +433,7 @@ config_read_vnc_params(int file, struct config_sesman *cs, struct list *param_n,
 }
 
 /******************************************************************************/
-int
+static int
 config_read_session_variables(int file, struct config_sesman *cs,
                               struct list *param_n, struct list *param_v)
 {
@@ -458,6 +460,67 @@ config_read_session_variables(int file, struct config_sesman *cs,
     return 0;
 }
 
+/******************************************************************************/
+struct config_sesman *
+config_read(const char *sesman_ini)
+{
+    struct config_sesman *cfg;
+    int all_ok = 0;
+
+    if ((cfg = g_new0(struct config_sesman, 1)) != NULL)
+    {
+        if ((cfg->sesman_ini = g_strdup(sesman_ini)) != NULL)
+        {
+            int fd;
+            if ((fd = g_file_open_ex(cfg->sesman_ini, 1, 0, 0, 0)) != -1)
+            {
+                struct list *sec;
+                struct list *param_n;
+                struct list *param_v;
+                sec = list_create();
+                sec->auto_free = 1;
+                file_read_sections(fd, sec);
+                param_n = list_create();
+                param_n->auto_free = 1;
+                param_v = list_create();
+                param_v->auto_free = 1;
+
+                /* read global config */
+                config_read_globals(fd, cfg, param_n, param_v);
+
+                /* read Xvnc/X11rdp/Xorg parameter list */
+                config_read_vnc_params(fd, cfg, param_n, param_v);
+                config_read_rdp_params(fd, cfg, param_n, param_v);
+                config_read_xorg_params(fd, cfg, param_n, param_v);
+
+                /* read security config */
+                config_read_security(fd, &(cfg->sec), param_n, param_v);
+
+                /* read session config */
+                config_read_sessions(fd, &(cfg->sess), param_n, param_v);
+
+                config_read_session_variables(fd, cfg, param_n, param_v);
+
+                /* cleanup */
+                list_delete(sec);
+                list_delete(param_v);
+                list_delete(param_n);
+                g_file_close(fd);
+                all_ok = 1;
+            }
+        }
+    }
+
+    if (!all_ok)
+    {
+        config_free(cfg);
+        cfg = NULL;
+    }
+
+    return cfg;
+}
+
+/******************************************************************************/
 void
 config_dump(struct config_sesman *config)
 {
@@ -468,6 +531,7 @@ config_dump(struct config_sesman *config)
     sc = &(config->sec);
 
     /* Global sesman configuration */
+    g_writeln("Filename:                     %s", config->sesman_ini);
     g_writeln("Global configuration:");
     g_writeln("    ListenAddress:            %s", config->listen_address);
     g_writeln("    ListenPort:               %s", config->listen_port);
@@ -567,16 +631,21 @@ config_dump(struct config_sesman *config)
     }
 }
 
+/******************************************************************************/
 void
 config_free(struct config_sesman *cs)
 {
-    g_free(cs->default_wm);
-    g_free(cs->reconnect_sh);
-    g_free(cs->auth_file_path);
-    list_delete(cs->rdp_params);
-    list_delete(cs->vnc_params);
-    list_delete(cs->xorg_params);
-    list_delete(cs->env_names);
-    list_delete(cs->env_values);
-    g_free(cs);
+    if (cs != NULL)
+    {
+        g_free(cs->sesman_ini);
+        g_free(cs->default_wm);
+        g_free(cs->reconnect_sh);
+        g_free(cs->auth_file_path);
+        list_delete(cs->rdp_params);
+        list_delete(cs->vnc_params);
+        list_delete(cs->xorg_params);
+        list_delete(cs->env_names);
+        list_delete(cs->env_values);
+        g_free(cs);
+    }
 }
