@@ -27,10 +27,6 @@
 #ifndef LIBSCP_H
 #define LIBSCP_H
 
-#if defined(HAVE_CONFIG_H)
-#include "config_ac.h"
-#endif
-
 #include "libscp_types.h"
 #include "libscp_commands.h"
 
