@@ -33,8 +33,8 @@
 extern struct config_sesman *g_cfg; /* in sesman.c */
 
 /******************************************************************************/
-void
-scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
+enum SCP_SERVER_STATES_E
+scp_v0_process(struct trans *t, struct SCP_SESSION *s)
 {
     int display = 0;
     tbus data;
@@ -53,15 +53,15 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
             if (1 == access_login_allowed(s->username))
             {
                 /* the user is member of the correct groups. */
-                scp_v0s_replyauthentication(c, errorcode);
+                scp_v0s_replyauthentication(t, errorcode);
                 LOG(LOG_LEVEL_INFO, "Access permitted for user: %s",
                     s->username);
                 /* g_writeln("Connection allowed"); */
             }
             else
             {
-                scp_v0s_replyauthentication(c, 32 + 3); /* all first 32 are reserved for PAM errors */
-                LOG(LOG_LEVEL_INFO, "Username okey but group problem for "
+                scp_v0s_replyauthentication(t, 32 + 3); /* all first 32 are reserved for PAM errors */
+                LOG(LOG_LEVEL_INFO, "Username okay but group problem for "
                     "user: %s", s->username);
                 /* g_writeln("user password ok, but group problem"); */
             }
@@ -71,7 +71,7 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
             /* g_writeln("username or password error"); */
             LOG(LOG_LEVEL_INFO, "Username or password error for user: %s",
                 s->username);
-            scp_v0s_replyauthentication(c, errorcode);
+            scp_v0s_replyauthentication(t, errorcode);
         }
     }
     else if (data)
@@ -123,18 +123,18 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
                 if (SCP_SESSION_TYPE_XVNC == s->type)
                 {
                     LOG( LOG_LEVEL_INFO, "starting Xvnc session...");
-                    display = session_start(data, SESMAN_SESSION_TYPE_XVNC, c, s);
+                    display = session_start(data, SESMAN_SESSION_TYPE_XVNC, s);
                 }
                 else if (SCP_SESSION_TYPE_XRDP == s->type)
                 {
                     LOG(LOG_LEVEL_INFO, "starting X11rdp session...");
-                    display = session_start(data, SESMAN_SESSION_TYPE_XRDP, c, s);
+                    display = session_start(data, SESMAN_SESSION_TYPE_XRDP, s);
                 }
                 else if (SCP_SESSION_TYPE_XORG == s->type)
                 {
                     /* type is SCP_SESSION_TYPE_XORG */
                     LOG(LOG_LEVEL_INFO, "starting Xorg session...");
-                    display = session_start(data, SESMAN_SESSION_TYPE_XORG, c, s);
+                    display = session_start(data, SESMAN_SESSION_TYPE_XORG, s);
                 }
                 /* if the session started up ok, auth_end will be called on
                    sig child */
@@ -148,19 +148,22 @@ scp_v0_process(struct SCP_CONNECTION *c, struct SCP_SESSION *s)
 
         if (display == 0)
         {
-            scp_v0s_deny_connection(c);
+            scp_v0s_deny_connection(t);
         }
         else
         {
-            scp_v0s_allow_connection(c, display, s->guid);
+            scp_v0s_allow_connection(t, display, s->guid);
         }
     }
     else
     {
-        scp_v0s_deny_connection(c);
+        LOG(LOG_LEVEL_INFO, "Username or password error for user: %s",
+            s->username);
+        scp_v0s_deny_connection(t);
     }
     if (do_auth_end)
     {
         auth_end(data);
     }
+    return SCP_SERVER_STATE_END;
 }
