@@ -27,14 +27,10 @@
 #ifndef SESMAN_H
 #define SESMAN_H
 
-#if defined(HAVE_CONFIG_H)
-#include "config_ac.h"
-#endif
 #include "arch.h"
 #include "parse.h"
 #include "os_calls.h"
 #include "log.h"
-#include "file_loc.h"
 #include "env.h"
 #include "auth.h"
 #include "config.h"
@@ -45,4 +41,16 @@
 
 #include "libscp.h"
 
+/*
+ * Close all file descriptors used by sesman.
+ *
+ * This is generally used after forking, to make sure the
+ * file descriptors used by the main process are not disturbed
+ *
+ * This call will also release all trans and SCP_SESSION objects
+ * held by sesman
+ */
+int
+sesman_close_all(void);
+
 #endif
