@@ -37,22 +37,21 @@ int main(int argc, char **argv)
 {
     char buf[256];
     struct SCP_SESSION *s;
-    struct SCP_CONNECTION *c;
+    struct trans *t;
     /*struct SCP_DISCONNECTED_SESSION ds;*/
     struct SCP_DISCONNECTED_SESSION *dsl;
     enum SCP_CLIENT_STATES_E e;
-    struct log_config log;
+    struct log_config *logging;
     int end;
     int scnt;
     int idx;
     int sel;
     int sock;
 
-    log.enable_syslog = 0;
-    log.log_level = LOG_LEVEL_DEBUG;
-    log.program_name = "sestest";
-    log.log_file = g_strdup("sestest.log");
-    log_start_from_param(&log);
+    logging = log_config_init_for_console(LOG_LEVEL_INFO, NULL);
+    log_start_from_param(logging);
+    log_config_free(logging);
+
     scp_init();
 
     sock = g_tcp_socket();
@@ -62,9 +61,9 @@ int main(int argc, char **argv)
     }
 
     s = scp_session_create();
-    c = scp_connection_create(sock);
+    t = scp_trans_create(sock);
 
-    if (0 != g_tcp_connect(sock, "localhost", "3350"))
+    if (0 != trans_connect(t, "localhost", "3350", 3000))
     {
         g_printf("error connecting");
         return 1;
@@ -105,7 +104,7 @@ int main(int argc, char **argv)
     s.errstr=0;*/
 
     end = 0;
-    e = scp_v1c_connect(c, s);
+    e = scp_v1c_connect(t, s);
 
     while (!end)
     {
@@ -117,7 +116,7 @@ int main(int argc, char **argv)
                 break;
             case SCP_CLIENT_STATE_SESSION_LIST:
                 g_printf("OK : session list needed\n");
-                e = scp_v1c_get_session_list(c, &scnt, &dsl);
+                e = scp_v1c_get_session_list(t, &scnt, &dsl);
                 break;
             case SCP_CLIENT_STATE_LIST_OK:
                 g_printf("OK : selecting a session:\n");
@@ -131,7 +130,7 @@ int main(int argc, char **argv)
                 }
 
                 sel = menuSelect(scnt);
-                e = scp_v1c_select_session(c, s, dsl[sel - 1].SID);
+                e = scp_v1c_select_session(t, s, dsl[sel - 1].SID);
                 g_printf("\n return: %d \n", e);
                 break;
             case SCP_CLIENT_STATE_RESEND_CREDENTIALS:
@@ -152,7 +151,7 @@ int main(int argc, char **argv)
                 }
 
                 scp_session_set_password(s, buf);
-                e = scp_v1c_resend_credentials(c, s);
+                e = scp_v1c_resend_credentials(t, s);
                 break;
             case SCP_CLIENT_STATE_CONNECTION_DENIED:
                 g_printf("ERR: connection denied: %s\n", s->errstr);
@@ -173,11 +172,9 @@ int main(int argc, char **argv)
         }
     }
 
-    g_tcp_close(sock);
     scp_session_destroy(s);
-    scp_connection_destroy(c);
-    /*free_stream(c.in_s);
-    free_stream(c.out_s);*/
+    trans_delete(t);
+    log_end();
 
     return 0;
 }
