@@ -36,8 +36,8 @@
  * @return 0 on success, 1 on error
  *
  */
-int DEFAULT_CC
-tcp_force_recv(int sck, char* data, int len);
+int
+tcp_force_recv(int sck, char *data, int len);
 
 /**
  *
@@ -48,8 +48,8 @@ tcp_force_recv(int sck, char* data, int len);
  * @return 0 on success, 1 on error
  *
  */
-int DEFAULT_CC
-tcp_force_send(int sck, char* data, int len);
+int
+tcp_force_send(int sck, char *data, int len);
 
 /**
  *
@@ -60,7 +60,7 @@ tcp_force_send(int sck, char* data, int len);
  * @return 0 on success, -1 on error
  *
  */
-int DEFAULT_CC
-tcp_bind(int sck, char* addr, char* port);
+int
+tcp_bind(int sck, char *addr, char *port);
 
 #endif
