@@ -241,12 +241,12 @@ xrdp_wm_ok_clicked(struct xrdp_bitmap *wnd)
             /* will copy these cause dialog gets freed */
             list_append_list_strdup(mod_data->names, wm->mm->login_names, 0);
             list_append_list_strdup(mod_data->values, wm->mm->login_values, 0);
-            xrdp_wm_set_login_mode(wm, 2);
+            xrdp_wm_set_login_state(wm, WMLS_START_CONNECT);
         }
     }
     else
     {
-        log_message(LOG_LEVEL_ERROR, "Combo is 0 - potential programming error");
+        LOG(LOG_LEVEL_WARNING, "Combo is NULL - potential programming error");
     }
 
     return 0;
@@ -254,18 +254,18 @@ xrdp_wm_ok_clicked(struct xrdp_bitmap *wnd)
 
 /*****************************************************************************/
 /**
-* This is an internal function in this file used to parse the domain 
-* information sent from the client. If the information starts 
-* with '_' the domain field contains the IP/DNS to connect to. 
-* If the domain field contains an additional '__' the char that 
-* follows this '__' is an index number of a preferred combo choice. 
-* Valid values for this choice is 0-9. But this function will only return 
+* This is an internal function in this file used to parse the domain
+* information sent from the client. If the information starts
+* with '_' the domain field contains the IP/DNS to connect to.
+* If the domain field contains an additional '__' the char that
+* follows this '__' is an index number of a preferred combo choice.
+* Valid values for this choice is 0-9. But this function will only return
 * index numbers between 0 and the max number of combo items -1.
-* Example: _192.168.1.2__1 result in a resultbuffer containing 
-* 192.168.1.2  and the return value will be 1. Meaning that 
-* index 1 is the preferred combo choice. 
-* 
-* Users can create shortcuts where this information is configured. These 
+* Example: _192.168.1.2__1 result in a resultbuffer containing
+* 192.168.1.2  and the return value will be 1. Meaning that
+* index 1 is the preferred combo choice.
+*
+* Users can create shortcuts where this information is configured. These
 * shortcuts simplifies login.
 * @param originalDomainInfo indata to this function
 * @param comboMax the max number of combo choices
@@ -293,39 +293,38 @@ xrdp_wm_parse_domain_information(char *originalDomainInfo, int comboMax,
     {
         /* we try to locate a number indicating what combobox index the user
          * prefer the information is loaded from domain field, from the client
-         * log_message(LOG_LEVEL_DEBUG, "domain contains _");
          * We must use valid chars in the domain name.
          * Underscore is a valid name in the domain.
-         * Invalid chars are ignored in microsoft client therefore we use '_' 
+         * Invalid chars are ignored in microsoft client therefore we use '_'
          * again. this sec '__' contains the split for index.*/
         pos = g_pos(&originalDomainInfo[1], "__");
         if (pos > 0)
         {
-            /* an index is found we try to use it
-            log_message(LOG_LEVEL_DEBUG, "domain contains index char __");*/
+            /* an index is found we try to use it */
+            LOG(LOG_LEVEL_DEBUG, "domain contains index char __");
             if (decode)
             {
                 g_memset(index, 0, 2);
                 /* we just accept values 0-9  (one figure) */
                 g_strncpy(index, &originalDomainInfo[pos + 3], 1);
                 comboxindex = g_htoi(index);
-                log_message(LOG_LEVEL_DEBUG,
-                            "index value as string: %s, as int: %d, max: %d",
-                            index, comboxindex, comboMax - 1);
+                LOG(LOG_LEVEL_DEBUG,
+                    "index value as string: %s, as int: %d, max: %d",
+                    index, comboxindex, comboMax - 1);
                 /* limit to max number of items in combo box */
                 if ((comboxindex > 0) && (comboxindex < comboMax))
                 {
-                    log_message(LOG_LEVEL_DEBUG, "domain contains a valid "
-                                "index number");
+                    LOG(LOG_LEVEL_DEBUG, "domain contains a valid "
+                        "index number");
                     ret = comboxindex; /* preferred index for combo box. */
                 }
             }
             /* pos limit the String to only contain the IP */
-            g_strncpy(resultBuffer, &originalDomainInfo[1], pos); 
+            g_strncpy(resultBuffer, &originalDomainInfo[1], pos);
         }
         else
         {
-            /* log_message(LOG_LEVEL_DEBUG, "domain does not contain _"); */
+            LOG(LOG_LEVEL_DEBUG, "domain does not contain _");
             g_strncpy(resultBuffer, &originalDomainInfo[1], 255);
         }
     }
@@ -446,8 +445,8 @@ xrdp_wm_show_edits(struct xrdp_wm *self, struct xrdp_bitmap *combo)
                     if (self->session->client_info->domain[0] == '_')
                     {
                         xrdp_wm_parse_domain_information(
-                                self->session->client_info->domain,
-                                combo->data_list->count, 0, resultIP);
+                            self->session->client_info->domain,
+                            combo->data_list->count, 0, resultIP);
                         g_strncpy(b->caption1, resultIP, 255);
                         b->edit_pos = g_mbstowcs(0, b->caption1, 0);
                     }
@@ -573,8 +572,8 @@ xrdp_wm_login_fill_in_combo(struct xrdp_wm *self, struct xrdp_bitmap *b)
 
     if (fd < 0)
     {
-        log_message(LOG_LEVEL_ERROR, "Could not read xrdp ini file %s",
-                    xrdp_ini);
+        LOG(LOG_LEVEL_ERROR, "Could not read xrdp ini file %s",
+            xrdp_ini);
         list_delete(sections);
         list_delete(section_names);
         list_delete(section_values);
@@ -711,15 +710,15 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
     /* if window title not specified, use hostname as default */
     if (globals->ls_title[0] == 0)
     {
-       g_gethostname(buf1, 256);
-       g_sprintf(buf, "Login to %s", buf1);
-       set_string(&self->login_window->caption1, buf);
+        g_gethostname(buf1, 256);
+        g_sprintf(buf, "Login to %s", buf1);
+        set_string(&self->login_window->caption1, buf);
     }
     else
     {
-       /*self->login_window->caption1 = globals->ls_title[0];*/
-       g_sprintf(buf, "%s", globals->ls_title);
-       set_string(&self->login_window->caption1, buf);
+        /*self->login_window->caption1 = globals->ls_title[0];*/
+        g_sprintf(buf, "%s", globals->ls_title);
+        set_string(&self->login_window->caption1, buf);
     }
 
     if (regular)
@@ -740,7 +739,7 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
                 g_snprintf(fileName, 255, "%s/%s",
                            XRDP_SHARE_PATH, globals->ls_background_image);
             }
-            log_message(LOG_LEVEL_DEBUG, "We try to load the following background file: %s", fileName);
+            LOG(LOG_LEVEL_DEBUG, "We try to load the following background file: %s", fileName);
             xrdp_bitmap_load(but, fileName, self->palette);
             but->parent = self->screen;
             but->owner = self->screen;
@@ -751,13 +750,17 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
 
         /* if logo image not specified, use default */
         if (globals->ls_logo_filename[0] == 0)
+        {
             g_snprintf(globals->ls_logo_filename, 255, "%s/xrdp_logo.bmp", XRDP_SHARE_PATH);
+        }
 
         /* logo image */
         but = xrdp_bitmap_create(4, 4, self->screen->bpp, WND_TYPE_IMAGE, self);
 
         if (self->screen->bpp <= 8)
+        {
             g_snprintf(globals->ls_logo_filename, 255, "%s/ad256.bmp", XRDP_SHARE_PATH);
+        }
 
         xrdp_bitmap_load(but, globals->ls_logo_filename, self->palette);
         but->parent = self->login_window;
@@ -820,9 +823,9 @@ xrdp_login_wnd_create(struct xrdp_wm *self)
     * We only perform this the first time for each connection.
     */
     combo->item_index = xrdp_wm_parse_domain_information(
-                self->session->client_info->domain,
-                combo->data_list->count, 1,
-                resultIP /* just a dummy place holder, we ignore */ );
+                            self->session->client_info->domain,
+                            combo->data_list->count, 1,
+                            resultIP /* just a dummy place holder, we ignore */ );
     xrdp_wm_show_edits(self, combo);
 
     return 0;
@@ -851,7 +854,9 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
     int   i;
 
     if (!config)
+    {
         return -1;
+    }
 
     globals = &config->cfg_globals;
 
@@ -871,7 +876,7 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
     globals->ls_btn_ok_x_pos = 150;
     globals->ls_btn_ok_y_pos = 300;
     globals->ls_btn_ok_width = 85;
-    globals->ls_btn_ok_height =30;
+    globals->ls_btn_ok_height = 30;
     globals->ls_btn_cancel_x_pos = 245;
     globals->ls_btn_cancel_y_pos = 300;
     globals->ls_btn_cancel_width = 85;
@@ -880,8 +885,8 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
     /* open xrdp.ini file */
     if ((fd = g_file_open(xrdp_ini)) < 0)
     {
-        log_message(LOG_LEVEL_ERROR,"load_config: Could not read "
-                    "xrdp.ini file %s", xrdp_ini);
+        LOG(LOG_LEVEL_ERROR, "load_config: Could not read "
+            "xrdp.ini file %s", xrdp_ini);
         return -1;
 
     }
@@ -896,8 +901,8 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
         list_delete(names);
         list_delete(values);
         g_file_close(fd);
-        log_message(LOG_LEVEL_ERROR,"load_config: Could not read globals "
-                    "section from xrdp.ini file %s", xrdp_ini);
+        LOG(LOG_LEVEL_ERROR, "load_config: Could not read globals "
+            "section from xrdp.ini file %s", xrdp_ini);
         return -1;
     }
 
@@ -911,117 +916,186 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
          */
 
         if (g_strncmp(n, "ini_version", 64) == 0)
+        {
             globals->ini_version = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "bitmap_cache", 64) == 0)
+        {
             globals->use_bitmap_cache = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "bitmap_compression", 64) == 0)
+        {
             globals->use_bitmap_compression = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "port", 64) == 0)
+        {
             globals->port = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "crypt_level", 64) == 0)
         {
             if (g_strcmp(v, "low") == 0)
+            {
                 globals->crypt_level = 1;
+            }
             else if (g_strcmp(v, "medium") == 0)
+            {
                 globals->crypt_level = 2;
+            }
             else
+            {
                 globals->crypt_level = 3;
+            }
         }
 
         else if (g_strncmp(n, "allow_channels", 64) == 0)
+        {
             globals->allow_channels = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "max_bpp", 64) == 0)
+        {
             globals->max_bpp = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "fork", 64) == 0)
+        {
             globals->fork = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "tcp_nodelay", 64) == 0)
+        {
             globals->tcp_nodelay = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "tcp_keepalive", 64) == 0)
+        {
             globals->tcp_keepalive = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "tcp_send_buffer_bytes", 64) == 0)
+        {
             globals->tcp_send_buffer_bytes = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "tcp_recv_buffer_bytes", 64) == 0)
+        {
             globals->tcp_recv_buffer_bytes = g_atoi(v);
+        }
 
         /* colors */
 
         else if (g_strncmp(n, "grey", 64) == 0)
+        {
             globals->grey = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "black", 64) == 0)
+        {
             globals->black = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "dark_grey", 64) == 0)
+        {
             globals->dark_grey = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "blue", 64) == 0)
+        {
             globals->blue = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "dark_blue", 64) == 0)
+        {
             globals->dark_blue = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "white", 64) == 0)
+        {
             globals->white = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "red", 64) == 0)
+        {
             globals->red = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "green", 64) == 0)
+        {
             globals->green = xrdp_wm_htoi(v);
+        }
 
         else if (g_strncmp(n, "background", 64) == 0)
+        {
             globals->background = xrdp_wm_htoi(v);
+        }
 
         /* misc stuff */
 
         else if (g_strncmp(n, "autorun", 255) == 0)
+        {
             g_strncpy(globals->autorun, v, 255);
+        }
 
         else if (g_strncmp(n, "hidelogwindow", 64) == 0)
+        {
             globals->hidelogwindow = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "require_credentials", 64) == 0)
+        {
             globals->require_credentials = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "bulk_compression", 64) == 0)
+        {
             globals->bulk_compression = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "new_cursors", 64) == 0)
+        {
             globals->new_cursors = g_text2bool(v);
+        }
 
         else if (g_strncmp(n, "nego_sec_layer", 64) == 0)
+        {
             globals->nego_sec_layer = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "allow_multimon", 64) == 0)
+        {
             globals->allow_multimon = g_text2bool(v);
+        }
 
-        else if (g_strncmp(n, "enable_token_login", 64) == 0) {
-            log_message(LOG_LEVEL_DEBUG, "Token login detection enabled x");
+        else if (g_strncmp(n, "enable_token_login", 64) == 0)
+        {
+            LOG(LOG_LEVEL_DEBUG, "Token login detection enabled x");
             globals->enable_token_login = g_text2bool(v);
         }
 
         /* login screen values */
         else if (g_strncmp(n, "ls_top_window_bg_color", 64) == 0)
+        {
             globals->ls_top_window_bg_color = HCOLOR(bpp, xrdp_wm_htoi(v));
+        }
 
         else if (g_strncmp(n, "ls_width", 64) == 0)
+        {
             globals->ls_width = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_height", 64) == 0)
+        {
             globals->ls_height = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_bg_color", 64) == 0)
+        {
             globals->ls_bg_color = HCOLOR(bpp, xrdp_wm_htoi(v));
+        }
 
         else if (g_strncmp(n, "ls_title", 255) == 0)
         {
@@ -1040,108 +1114,136 @@ load_xrdp_config(struct xrdp_config *config, const char *xrdp_ini, int bpp)
             globals->ls_background_image[255] = 0;
         }
         else if (g_strncmp(n, "ls_logo_x_pos", 64) == 0)
+        {
             globals->ls_logo_x_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_logo_y_pos", 64) == 0)
+        {
             globals->ls_logo_y_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_label_x_pos", 64) == 0)
+        {
             globals->ls_label_x_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_label_width", 64) == 0)
+        {
             globals->ls_label_width = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_input_x_pos", 64) == 0)
+        {
             globals->ls_input_x_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_input_width", 64) == 0)
+        {
             globals->ls_input_width = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_input_y_pos", 64) == 0)
+        {
             globals->ls_input_y_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_ok_x_pos", 64) == 0)
+        {
             globals->ls_btn_ok_x_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_ok_y_pos", 64) == 0)
+        {
             globals->ls_btn_ok_y_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_ok_width", 64) == 0)
+        {
             globals->ls_btn_ok_width = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_ok_height", 64) == 0)
+        {
             globals->ls_btn_ok_height = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_cancel_x_pos", 64) == 0)
+        {
             globals->ls_btn_cancel_x_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_cancel_y_pos", 64) == 0)
+        {
             globals->ls_btn_cancel_y_pos = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_cancel_width", 64) == 0)
+        {
             globals->ls_btn_cancel_width = g_atoi(v);
+        }
 
         else if (g_strncmp(n, "ls_btn_cancel_height", 64) == 0)
+        {
             globals->ls_btn_cancel_height = g_atoi(v);
+        }
     }
 
-#if 0
-    g_writeln("ini_version:             %d", globals->ini_version);
-    g_writeln("use_bitmap_cache:        %d", globals->use_bitmap_cache);
-    g_writeln("use_bitmap_compression:  %d", globals->use_bitmap_compression);
-    g_writeln("port:                    %d", globals->port);
-    g_writeln("crypt_level:             %d", globals->crypt_level);
-    g_writeln("allow_channels:          %d", globals->allow_channels);
-    g_writeln("max_bpp:                 %d", globals->max_bpp);
-    g_writeln("fork:                    %d", globals->fork);
-    g_writeln("tcp_nodelay:             %d", globals->tcp_nodelay);
-    g_writeln("tcp_keepalive:           %d", globals->tcp_keepalive);
-    g_writeln("tcp_send_buffer_bytes:   %d", globals->tcp_send_buffer_bytes);
-    g_writeln("tcp_recv_buffer_bytes:   %d", globals->tcp_recv_buffer_bytes);
-    g_writeln("new_cursors:             %d", globals->new_cursors);
-    g_writeln("allow_multimon:          %d", globals->allow_multimon);
+    LOG(LOG_LEVEL_DEBUG, "ini_version:             %d", globals->ini_version);
+    LOG(LOG_LEVEL_DEBUG, "use_bitmap_cache:        %d", globals->use_bitmap_cache);
+    LOG(LOG_LEVEL_DEBUG, "use_bitmap_compression:  %d", globals->use_bitmap_compression);
+    LOG(LOG_LEVEL_DEBUG, "port:                    %d", globals->port);
+    LOG(LOG_LEVEL_DEBUG, "crypt_level:             %d", globals->crypt_level);
+    LOG(LOG_LEVEL_DEBUG, "allow_channels:          %d", globals->allow_channels);
+    LOG(LOG_LEVEL_DEBUG, "max_bpp:                 %d", globals->max_bpp);
+    LOG(LOG_LEVEL_DEBUG, "fork:                    %d", globals->fork);
+    LOG(LOG_LEVEL_DEBUG, "tcp_nodelay:             %d", globals->tcp_nodelay);
+    LOG(LOG_LEVEL_DEBUG, "tcp_keepalive:           %d", globals->tcp_keepalive);
+    LOG(LOG_LEVEL_DEBUG, "tcp_send_buffer_bytes:   %d", globals->tcp_send_buffer_bytes);
+    LOG(LOG_LEVEL_DEBUG, "tcp_recv_buffer_bytes:   %d", globals->tcp_recv_buffer_bytes);
+    LOG(LOG_LEVEL_DEBUG, "new_cursors:             %d", globals->new_cursors);
+    LOG(LOG_LEVEL_DEBUG, "allow_multimon:          %d", globals->allow_multimon);
 
-    g_writeln("grey:                    %d", globals->grey);
-    g_writeln("black:                   %d", globals->black);
-    g_writeln("dark_grey:               %d", globals->dark_grey);
-    g_writeln("blue:                    %d", globals->blue);
-    g_writeln("dark_blue:               %d", globals->dark_blue);
-    g_writeln("white:                   %d", globals->white);
-    g_writeln("red:                     %d", globals->red);
-    g_writeln("green:                   %d", globals->green);
-    g_writeln("background:              %d", globals->background);
+    LOG(LOG_LEVEL_DEBUG, "grey:                    %d", globals->grey);
+    LOG(LOG_LEVEL_DEBUG, "black:                   %d", globals->black);
+    LOG(LOG_LEVEL_DEBUG, "dark_grey:               %d", globals->dark_grey);
+    LOG(LOG_LEVEL_DEBUG, "blue:                    %d", globals->blue);
+    LOG(LOG_LEVEL_DEBUG, "dark_blue:               %d", globals->dark_blue);
+    LOG(LOG_LEVEL_DEBUG, "white:                   %d", globals->white);
+    LOG(LOG_LEVEL_DEBUG, "red:                     %d", globals->red);
+    LOG(LOG_LEVEL_DEBUG, "green:                   %d", globals->green);
+    LOG(LOG_LEVEL_DEBUG, "background:              %d", globals->background);
 
-    g_writeln("autorun:                 %s", globals->autorun);
-    g_writeln("hidelogwindow:           %d", globals->hidelogwindow);
-    g_writeln("require_credentials:     %d", globals->require_credentials);
-    g_writeln("bulk_compression:        %d", globals->bulk_compression);
-    g_writeln("new_cursors:             %d", globals->new_cursors);
-    g_writeln("nego_sec_layer:          %d", globals->nego_sec_layer);
-    g_writeln("allow_multimon:          %d", globals->allow_multimon);
-    g_writeln("enable_token_login:      %d", globals->enable_token_login)
+    LOG(LOG_LEVEL_DEBUG, "autorun:                 %s", globals->autorun);
+    LOG(LOG_LEVEL_DEBUG, "hidelogwindow:           %d", globals->hidelogwindow);
+    LOG(LOG_LEVEL_DEBUG, "require_credentials:     %d", globals->require_credentials);
+    LOG(LOG_LEVEL_DEBUG, "bulk_compression:        %d", globals->bulk_compression);
+    LOG(LOG_LEVEL_DEBUG, "new_cursors:             %d", globals->new_cursors);
+    LOG(LOG_LEVEL_DEBUG, "nego_sec_layer:          %d", globals->nego_sec_layer);
+    LOG(LOG_LEVEL_DEBUG, "allow_multimon:          %d", globals->allow_multimon);
+    LOG(LOG_LEVEL_DEBUG, "enable_token_login:      %d", globals->enable_token_login);
 
-    g_writeln("ls_top_window_bg_color:  %x", globals->ls_top_window_bg_color);
-    g_writeln("ls_width:                %d", globals->ls_width);
-    g_writeln("ls_height:               %d", globals->ls_height);
-    g_writeln("ls_bg_color:             %x", globals->ls_bg_color);
-    g_writeln("ls_title:                %s", globals->ls_title);
-    g_writeln("ls_logo_filename:        %s", globals->ls_logo_filename);
-    g_writeln("ls_logo_x_pos:           %d", globals->ls_logo_x_pos);
-    g_writeln("ls_logo_y_pos:           %d", globals->ls_logo_y_pos);
-    g_writeln("ls_label_x_pos:          %d", globals->ls_label_x_pos);
-    g_writeln("ls_label_width:          %d", globals->ls_label_width);
-    g_writeln("ls_input_x_pos:          %d", globals->ls_input_x_pos);
-    g_writeln("ls_input_width:          %d", globals->ls_input_width);
-    g_writeln("ls_input_y_pos:          %d", globals->ls_input_y_pos);
-    g_writeln("ls_btn_ok_x_pos:         %d", globals->ls_btn_ok_x_pos);
-    g_writeln("ls_btn_ok_y_pos:         %d", globals->ls_btn_ok_y_pos);
-    g_writeln("ls_btn_ok_width:         %d", globals->ls_btn_ok_width);
-    g_writeln("ls_btn_ok_height:        %d", globals->ls_btn_ok_height);
-    g_writeln("ls_btn_cancel_x_pos:     %d", globals->ls_btn_cancel_x_pos);
-    g_writeln("ls_btn_cancel_y_pos:     %d", globals->ls_btn_cancel_y_pos);
-    g_writeln("ls_btn_cancel_width:     %d", globals->ls_btn_cancel_width);
-    g_writeln("ls_btn_cancel_height:    %d", globals->ls_btn_cancel_height);
-#endif
+    LOG(LOG_LEVEL_DEBUG, "ls_top_window_bg_color:  %x", globals->ls_top_window_bg_color);
+    LOG(LOG_LEVEL_DEBUG, "ls_width:                %d", globals->ls_width);
+    LOG(LOG_LEVEL_DEBUG, "ls_height:               %d", globals->ls_height);
+    LOG(LOG_LEVEL_DEBUG, "ls_bg_color:             %x", globals->ls_bg_color);
+    LOG(LOG_LEVEL_DEBUG, "ls_title:                %s", globals->ls_title);
+    LOG(LOG_LEVEL_DEBUG, "ls_logo_filename:        %s", globals->ls_logo_filename);
+    LOG(LOG_LEVEL_DEBUG, "ls_logo_x_pos:           %d", globals->ls_logo_x_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_logo_y_pos:           %d", globals->ls_logo_y_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_label_x_pos:          %d", globals->ls_label_x_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_label_width:          %d", globals->ls_label_width);
+    LOG(LOG_LEVEL_DEBUG, "ls_input_x_pos:          %d", globals->ls_input_x_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_input_width:          %d", globals->ls_input_width);
+    LOG(LOG_LEVEL_DEBUG, "ls_input_y_pos:          %d", globals->ls_input_y_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_ok_x_pos:         %d", globals->ls_btn_ok_x_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_ok_y_pos:         %d", globals->ls_btn_ok_y_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_ok_width:         %d", globals->ls_btn_ok_width);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_ok_height:        %d", globals->ls_btn_ok_height);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_cancel_x_pos:     %d", globals->ls_btn_cancel_x_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_cancel_y_pos:     %d", globals->ls_btn_cancel_y_pos);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_cancel_width:     %d", globals->ls_btn_cancel_width);
+    LOG(LOG_LEVEL_DEBUG, "ls_btn_cancel_height:    %d", globals->ls_btn_cancel_height);
 
     list_delete(names);
     list_delete(values);
