@@ -228,6 +228,7 @@ xrdp_wm_load_pointer(struct xrdp_wm *self, char *file_name, char *data,
                      char *mask, int *x, int *y)
 {
     int fd;
+    int len;
     int bpp;
     int w;
     int h;
@@ -256,8 +257,16 @@ xrdp_wm_load_pointer(struct xrdp_wm *self, char *file_name, char *data,
         return 1;
     }
 
-    g_file_read(fd, fs->data, 8192);
+    len = g_file_read(fd, fs->data, 8192);
     g_file_close(fd);
+    if (len <= 0)
+    {
+        LOG(LOG_LEVEL_ERROR, "xrdp_wm_load_pointer: read error from file [%s]",
+            file_name);
+        xstream_free(fs);
+        return 1;
+    }
+    fs->end = fs->data + len;
     in_uint8s(fs, 6);
     in_uint8(fs, w);
     in_uint8(fs, h);
